/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package examples.invoice.applications;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

/**
 * Observes an InputStream and redirect all read data to the standard
 * OutputStream (System.out).
 * @author	P. Dechamboux
 */
class ObsInputStream extends InputStream {
	private InputStream in;

	public static InputStream observes(String cmdfile) {
		try {
			return new ObsInputStream(new FileInputStream(new File(cmdfile)));
		} catch (FileNotFoundException e) {
			System.err.println("The commands file cannot be accessed!");
			System.exit(-1);
			return null;
		}
	}

	public ObsInputStream(InputStream in) {
		this.in = in;
	}

	public int available() throws IOException {
		return in.available();
	}

	public void close() throws IOException {
		in.close();
	}

	public void mark(int readlimit) {
		in.mark(readlimit);
	}

	public boolean markSupported() {
		return in.markSupported();
	}

	public int read() throws IOException {
		int c;
		c = in.read();
		System.out.write(c);
		return c;
	}

	public int read(byte[] b) throws IOException {
		int size = in.read(b);
		System.out.write(b, 0, size);
		return size;
	}

	public int read(byte[] b, int off, int len) throws IOException {
		int size = in.read(b, off, len);
		System.out.write(b, off, size);
		return size;
	}

	public void reset() throws IOException {
		in.reset();
	}

	public long skip(long n) throws IOException {
		return in.skip(n);
	}
}
