/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package examples.invoice.persistclass;

import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNamingContext;

/**
 * @author	P. Dechamboux
 */
public class ProdUnits implements PAccessor, ProdUnitsAccessor {
	private PName productPn;
	private Product product;
	private int quantity;
	private PBinding pBinding;
	private ProdUnitsHelper prodUnitsHelper;
    private PNamingContext productNC;

	public PBinding getPBinding() {
		return pBinding;
	}

	/**
	 * Implements lazy dereferencing of address field.
	 */
	private Product getProduct() throws PException {
		try {
			if (product == null)
				product = prodUnitsHelper.getProductHelper().getProduct(
					pBinding.getPClassMapping().getPMapper(), productPn);
		} catch (Exception e) {
			e.printStackTrace();
			throw new PException(e, "Cannot getProduct within ProdUnits!");
		}
		return product;
	}

	/**
	 * Creates a new Address.
	 */
	public ProdUnits(ProdUnitsHelper puh, Object conn, PBinding pb, PName pn, int quant) throws PException {
		prodUnitsHelper = puh;
		pBinding = pb;
        productNC = (PNamingContext) pBinding.getPClassMapping().getPNameCoder("product");
		productPn = pn;
		quantity = quant;
		pBinding.export(conn);
		pBinding.write(conn, this);
	}

	/**
	 * Creates an Address that is already persistent.
	 */
	public ProdUnits(ProdUnitsHelper puh, Object conn, PBinding pb) throws PException {
		prodUnitsHelper = puh;
		pBinding = pb;
        productNC = (PNamingContext) pBinding.getPClassMapping().getPNameCoder("product");
		pb.read(conn, this);
	}

	/**
	 * Turns this persistent object into a non-persistent one.
	 */
	public void delete(Object conn) throws PException {
		pBinding.unexport(conn);
		pBinding.write(conn, this);
		pBinding = null;
	}

	/**
	 * Yields a printable version a the full ProdUnits.
	 */
	public String toPrintable() throws PException {
		return "ID: [" + pBinding.getPName().encodeString() + "] - VALUE: [product: [" + getProduct().toPrintable() + "], quantity: " + quantity + "]";
	}

	// IMPLEMENTATION OF PAccessor INTERFACE

	public Object getMemoryInstance() {
		return this;
	}

	// IMPLEMENTATION OF AddressAccessor INTERFACE (generated by JORM)

	// Accessors to the product field
	public void paSetProduct(PName val, Object conn) throws PException {
		productPn = productNC.resolve(conn, val);
	}

	public PName paGetProduct(Object conn) throws PException {
		return productNC.export(conn, productPn);
	}

	// Accessors to the quantity field
	public void paSetQuantity(int val) throws PException {
		quantity = val;
	}

	public int paGetQuantity() throws PException {
		return quantity;
	}
}
