/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.facility.naming.basidir;

import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.lib.BasicPName;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class BasidName extends BasicPName implements PName {
	protected Object o = null;
	protected long p = 0;

	public BasidName(Object o, BasidBinder binder) throws PExceptionNaming {
		this.pnc = binder;
		if (o instanceof BasidName) {
			this.o = ((BasidName) o).o;
			this.p = ((BasidName) o).p;

		} else if (o instanceof PName) {
			switch (binder.codingType) {
			case PNamingContext.CTBYTE:
				p = ((PName) o).encodeByte();
				break;
			case PNamingContext.CTOBYTE:
				o = ((PName) o).encodeObyte();
				break;
			case PNamingContext.CTCHAR:
				p = ((PName) o).encodeChar();
				break;
			case PNamingContext.CTOCHAR:
				o = ((PName) o).encodeOchar();
				break;
			case PNamingContext.CTINT:
				p = ((PName) o).encodeInt();
				break;
			case PNamingContext.CTOINT:
				o = ((PName) o).encodeOint();
				break;
			case PNamingContext.CTLONG:
				p = ((PName) o).encodeLong();
				break;
			case PNamingContext.CTOLONG:
				o = ((PName) o).encodeOlong();
				break;
			case PNamingContext.CTSHORT:
				p = ((PName) o).encodeShort();
				break;
			case PNamingContext.CTOSHORT:
				o = ((PName) o).encodeOshort();
				break;
			case PNamingContext.CTDATE:
				o = ((PName) o).encodeDate();
				break;
			case PNamingContext.CTSTRING:
				o = ((PName) o).encodeString();
				break;
			case PNamingContext.CTBYTEARRAY:
				o = ((PName) o).encode();
				break;
			case PNamingContext.CTCHARARRAY:
				o = ((PName) o).encodeCharArray();
				break;
			default:
				throw new PExceptionNaming(
						"BasidName does not support this coding type");
			}

		} else {
			switch (binder.codingType) {
			case PNamingContext.CTBYTE:
				if (o == null) {
					throw new PExceptionNaming("Cannot initalize BasidName with null value (Byte object value expected)");
				}
				this.p = ((Byte) o).byteValue();
				break;
			case PNamingContext.CTCHAR:
				if (o == null) {
					throw new PExceptionNaming("Cannot initalize BasidName with null value (Character object value expected)");
				}
				this.p = ((Character) o).charValue();
				break;
			case PNamingContext.CTINT:
				if (o == null) {
					throw new PExceptionNaming("Cannot initalize BasidName with null value (Integer object value expected)");
				}
				this.p = ((Integer) o).intValue();
				break;
			case PNamingContext.CTLONG:
				if (o == null) {
					throw new PExceptionNaming("Cannot initalize BasidName with null value (Long object value expected)");
				}
				this.p = ((Long) o).longValue();
				break;
			case PNamingContext.CTSHORT:
				if (o == null) {
					throw new PExceptionNaming("Cannot initalize BasidName with null value (Short object value expected)");
				}
				this.p = ((Short) o).shortValue();
				break;
			default:
				this.o = o;
			}
		}
	}

	public BasidName(long p, BasidBinder binder) {
		this.pnc = binder;
		this.p = p;
	}

	// IMPLEMENTATION OF METHODS OF THE Object INTERFACE

	public boolean equals(Object obj) {
		return (this == obj)
				|| ((obj instanceof BasidName)
				&& pnc == ((BasidName) obj).pnc
				&& (o != null
				? o.equals(((BasidName) obj).o)
				: p == ((BasidName) obj).p));
	}

	public int hashCode() {
		if (o != null)
			return o.hashCode();
		return (int) p;
	}

	public String toString() {
		switch (((BasidBinder) pnc).codingType) {
		case PNamingContext.CTBYTE:
			return super.toString() + " / type=byte / value=" + p;
		case PNamingContext.CTCHAR:
			return super.toString() + " / type=char / value=" + (char) p;
		case PNamingContext.CTINT:
			return super.toString() + " / type=int / value=" + p;
		case PNamingContext.CTLONG:
			return super.toString() + " / type=long / value=" + p;
		case PNamingContext.CTSHORT:
			return super.toString() + " / type=short / value=" + p;
		default:
			return super.toString() + " / type=" + ((BasidBinder) pnc).codingType
					+ " / value=" + o;
		}
	}

	// IMPLEMENTATION OF METHODS OF THE PName INTERFACE

	public boolean isNull() {
		switch (((BasidBinder) pnc).codingType) {
		case PNamingContext.CTBYTE:
		case PNamingContext.CTCHAR:
		case PNamingContext.CTINT:
		case PNamingContext.CTLONG:
		case PNamingContext.CTSHORT:
			return p == ((BasidBinder) pnc).nullValue;
		default:
			return o == ((BasidBinder) pnc).nullOValue;
		}
	}
}
