/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.facility.naming.generator;

import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PException;

/**
 * Provides a means to allocates long identifiers generators.
 * @author	P. Dechamboux
 */
public interface LongGenMgr {
	/**
	 * Retrieves the mapper associated with this LongGenMgr.
	 */
	PMapper getPMapper();

	/**
	 * Initializes a long generator manager by mapping relevant classes
	 * to the given mapper, and setting up relevant initial persistent
	 * information.
	 */
	void init(PMapper pm, byte clact) throws PException;

	/**
	 * Retrieves a long generator with the given name. If it does not
	 * exist, creates it.
	 */
	LongGen getLongGen(String lgname) throws PException;
}
