/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.generator.lib;

import org.apache.velocity.app.VelocityEngine;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

/**
 * This class contains the common elements for generating jorm objects (mapping,
 * binding, accessors).
 */
public abstract class CommonGenerator implements Loggable {
	/**
	 * The base directory where to find velocity template.
	 */
	public static final String MAPPER_TEMPLATE_DIR =
			"org/objectweb/jorm/mapper/";
	public static final String GEN_TEMPLATE_DIR =
			"org/objectweb/jorm/generator/lib/";

	/**
	 * The logger instance which logs the traces of the generator.
	 */
	protected Logger logger = null;

	/**
	 * The logger factory instance which permits to fetch Logger instance.
	 */
	protected LoggerFactory loggerFactory = null;

	protected VelocityEngine velocityEngine = null;

	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLoggerFactory(LoggerFactory lf) {
		if (logger == null && lf != null)
			logger = lf.getLogger(getClass().getName());
		loggerFactory = lf;
	}

	public void setLogger(Logger l) {
		logger = l;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}

	// ABSTRACT METHODS

	public void setVelocityEngine(VelocityEngine ve) {
		velocityEngine = ve;
	}

	public String[][] getTemplateLibraries() {
		return new String[0][];
	}
}
