/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.fos.generator;

import org.apache.velocity.context.Context;

import org.objectweb.jorm.generator.api.MOP;
import org.objectweb.jorm.generator.lib.CommonGenerator;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

/**
 * @author	P. Dechamboux
 */
public class FosBindingMOP implements MOP, Loggable {
	protected Logger logger = null;
	protected LoggerFactory loggerFactory = null;

	public void initContext(Context ctx) {
		ctx.put("bindingMOPTools", this);
	}

	public String[] getTemplateLibraries() {
		String[] libs = {CommonGenerator.MAPPER_TEMPLATE_DIR + "fos/generator/FosBinding.vm"};
		return libs;
	}

	public String getSerRead(PType type)
			throws Exception {
		return "read" + getSer(type);
	}

	public String getSerWrite(PType type)
			throws Exception {
		return "write" + getSer(type);
	}

	private String getSer(PType type) {
		switch (type.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return "Boolean";
		case PType.TYPECODE_BYTE:
			return "Byte";
		case PType.TYPECODE_CHAR:
			return "Char";
		case PType.TYPECODE_DOUBLE:
			return "Double";
		case PType.TYPECODE_FLOAT:
			return "Float";
		case PType.TYPECODE_INT:
			return "Int";
		case PType.TYPECODE_LONG:
			return "Long";
		case PType.TYPECODE_SHORT:
			return "Short";
		default:
			return "Object";
		}
	}

	public String getSerObjectCast(PType type) {
		switch (type.getTypeCode()) {
		case PType.TYPECODE_OBJBOOLEAN:
			return "(Boolean) ";
		case PType.TYPECODE_OBJBYTE:
			return "(Byte) ";
		case PType.TYPECODE_OBJCHAR:
			return "(Character) ";
		case PType.TYPECODE_OBJDOUBLE:
			return "(Double) ";
		case PType.TYPECODE_OBJFLOAT:
			return "(Float) ";
		case PType.TYPECODE_OBJINT:
			return "(Integer) ";
		case PType.TYPECODE_OBJLONG:
			return "(Long) ";
		case PType.TYPECODE_OBJSHORT:
			return "(Short) ";
		case PType.TYPECODE_STRING:
			return "(String) ";
		case PType.TYPECODE_DATE:
			return "(Date) ";
		case PType.TYPECODE_SERIALIZED:
			return "(java.io.Serializable) ";
		case PType.TYPECODE_BIGDECIMAL:
			return "(java.math.BigDecimal) ";
		case PType.TYPECODE_BYTEARRAY:
			return "(byte[]) ";
		case PType.TYPECODE_CHARARRAY:
			return "(char[]) ";
		default:
			return "";
		}
	}

	// IMPLEMENTATION OF METHDOS FROM THE Loggable INTERFACE

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}

	public void setLogger(Logger l) {
		logger = l;
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
		if (loggerFactory != null && logger == null)
			loggerFactory.getLogger(getClass().getName());
	}
}
