/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.fos.generator;

import org.objectweb.jorm.generator.api.MOPFactory;
import org.objectweb.jorm.generator.api.MOP;
import org.objectweb.jorm.compiler.lib.JormCompilerParameter;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.metainfo.api.Mapping;

/**
 * @author	P. Dechamboux
 */
public class FosMOPFactory implements MOPFactory {
	/**
	 * This method must return a new instance of a MappingMOP
	 */
	public MOP createMappingMOP(JormCompilerParameter cp, String mappername) throws PException {
		return new FosMappingMOP();
	}

	/**
	 * This method must return a new instance of a BindingMOP
	 */
	public MOP createBindingMOP(JormCompilerParameter cp, String mappername) throws PException {
		return new FosBindingMOP();
	}

	/**
	 * Creates a generation context that can be shared between binding and
	 * mapping generation for a particular class.
	 */
	public Object createClassGenerationContext(Mapping mo) throws PException {
		return null;
	}
}
