/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.adapter;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PException;

/**
 * @author P. Dechamboux
 */
public class MckoiAdapter extends BasicRdbAdapter {

	public MckoiAdapter() {
		super();
	}

	public String getSqlType(PType pt, boolean usedInPK, int size) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("No Sql Type name associated to a null PType");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_STRING:
			if (size == PType.NOSIZE) {
				return "CLOB";
			}
			return "VARCHAR(" + size + ")";
		case PType.TYPECODE_BYTEARRAY:
		case PType.TYPECODE_SERIALIZED:
			return "BLOB";
		default:
			return super.getSqlType(pt, usedInPK, size);
		}
	}

	public String getSqlGetterName(PType pt, String resultSet, int idx, String sql)
			throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("Invalid parameter");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_OBJBYTE:
			return "new Byte(" + resultSet + ".getByte(" + idx + "))";
		case PType.TYPECODE_OBJSHORT:
			return "new Short(" + resultSet + ".getShort(" + idx + "))";
		default:
			return super.getSqlGetterName(pt, resultSet, idx, sql);
		}
	}

	public String getFirstLocateExpression(String substring, String instring)
			throws PException {
		throw new PException(
				"SQL function not supported in McKoi: firstLocate for substring "
				+ substring + " in " + instring);
	}

	public String getIndexedLocateExpression(String instring,
											 String substring,
											 String fromIndex)
			throws PException {
		throw new PException(
				"SQL function not supported in McKoi: indexedLocate for substring "
				+ substring + " in " + instring + " from index " + fromIndex);
	}
}
