/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.mapper.rdb.adapter;

import java.util.List;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class Oracle8Adapter extends OracleAdapter {

	public String getQuery(String selectClause, List jts, String whereclause) {
		StringBuffer sb = new StringBuffer();
		sb.append("SELECT ");
		sb.append(selectClause);
		sb.append(" FROM ");
		String sep = "";
		for (int i = 0; i < jts.size(); i++) {
			sep = getFromClause((JoinedTable) jts.get(i), sb, sep);
		}
		if (whereclause != null && whereclause.length() > 0) {
			sb.append(" WHERE ");
			sb.append(whereclause);
			sep = " AND ";
		}
		else {
			sep = " WHERE ";
		}
		for (int i = 0; i < jts.size(); i++) {
			sep = getWhereClause((JoinedTable) jts.get(i), sb, sep);
		}
		return sb.toString();
	}

	protected String getFromClause(JoinedTable jt, StringBuffer sb, String sep) {
		sb.append(sep);
		sb.append(jt.tableName);
		if (jt.alias != null && jt.alias.length() > 0) {
			writeTableAlias(jt.alias, sb);
		}
		sep = ", ";
		for (int i = 0; i < jt.joinedTables.size(); i++) {
			getFromClause((JoinedTable) jt.joinedTables.get(i), sb, sep);
		}
		return sep;
	}

	protected String getWhereClause(JoinedTable jt,
									StringBuffer sb,
									String sep) {
		for (int i = 0; i < jt.joins.size(); i++) {
			JoinedTable.Join jtj = (JoinedTable.Join) jt.joins.get(i);
			JoinedTable jtjt = (JoinedTable) jt.joinedTables.get(i);
			for (int j = 0; j < jtj.colT1.size(); j++) {
				sb.append(sep);
				sep = " AND ";
				sb.append('(');
				if (jt.alias != null && jt.alias.length() > 0) {
					sb.append(jt.alias);
				}
				else {
					sb.append(jt.tableName);
				}
				sb.append('.');
				sb.append(jtj.colT1.get(j));
				sb.append(" = ");
				if (jtjt.alias != null && jtjt.alias.length() > 0) {
					sb.append(jtjt.alias);
				}
				else {
					sb.append(jtjt.tableName);
				}
				sb.append('.');
				sb.append(jtj.colT2.get(j));
				sb.append(" (+))");
			}
			getWhereClause((JoinedTable) jt.joinedTables.get(i), sb, sep);
		}
		return sep;
	}
}
