/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.generator;

import org.objectweb.jorm.generator.lib.CommonHelper;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.mapper.rdb.adapter.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.RdbAdapterFactory;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbPrimitiveElementMapping;

import java.util.Map;
import java.util.HashMap;

/**
 * This helper contains all common tool methods shared by the RdbBindingMOP and
 * the RdbMappingMOP.
 * @author Sebastien Chassande-Barrioz
 */
public class RdbCommonHelper extends CommonHelper {
	private Map rdbAdapters = new HashMap();

	/**
	 * Retrieves the adapter associated with the given mapper.
	 * @param mappername    The name of the corresponding mapper.
	 * @return  The RDB adapter.
	 * @throws org.objectweb.jorm.api.PException
	 */
	protected RdbAdapter getRdbAdapter(String mappername) throws PException {
		RdbAdapter res = (RdbAdapter) rdbAdapters.get(mappername);
		if (res != null) {
			return res;
		}
		String dbn;
		if (mappername.indexOf(".") == -1)
			dbn = RdbAdapterFactory.DATABASE_NAME_JDBC;
		else
			dbn = mappername.substring(mappername.indexOf(".") + 1);
		res = RdbAdapterFactory.getTypeConverter(dbn);
		rdbAdapters.put(mappername, res);
		return res;
	}

	public String getSqlType(RdbPrimitiveElementMapping pem,
							 RdbAdapter adapter,
							 boolean usedInPk) throws PException {
		String userSQLType = pem.getType();
		if (userSQLType == null || userSQLType.length() == 0) {
			return adapter.getSqlType(
					((PrimitiveElement) pem.getLinkedMO()).getType(), usedInPk,
					((PrimitiveElement) pem.getLinkedMO()).getSize());
		} else {
			return userSQLType;
		}
	}
}
