/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.lib;

import org.objectweb.jorm.api.PException;

import javax.sql.DataSource;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Connection;
import java.io.PrintWriter;

/**
 * Specifies the characteristics of the requested connections.
 * @author	P. Dechamboux
 */
public class ConnectionSpecJDBC implements DataSource {
	String url;
	String user = null;
	String passwd = null;
	PrintWriter pw;
	int timeout = 0;

	/**
	 * Construct a ConnectionSpecJDBC knowing the DB url.
	 */
	public ConnectionSpecJDBC(String url, String driver) throws PException {
		this.url = url;
		try {
			Class.forName(driver);
		} catch (ClassNotFoundException e) {
			throw new PException(e, "Unable to load JDBC driver class: " + driver);
		}
	}

	/**
	 * Construct a ConnectionSpecJDBC knowing the DB url, user, and password.
	 */
	public ConnectionSpecJDBC(String url, String driver, String user, String passwd)
			throws PException {
		this(url, driver);
		this.user = user;
		this.passwd = passwd;
	}


	// IMPLEMENTATION OF THE DataSource INTERFACE //
	//--------------------------------------------//

	public Connection getConnection() throws SQLException {
		if (user != null && passwd != null) {
			return DriverManager.getConnection(url, user, passwd);
		} else {
			return DriverManager.getConnection(url);
		}
	}

	public Connection getConnection(String username, String password)
			throws SQLException {
		return DriverManager.getConnection(url, username, password);
	}

	public PrintWriter getLogWriter() throws SQLException {
		return pw;
	}

	public void setLogWriter(PrintWriter out) throws SQLException {
		pw = out;
	}

	public void setLoginTimeout(int seconds) throws SQLException {
		timeout = seconds;
	}

	public int getLoginTimeout() throws SQLException {
		return timeout;
	}
}
