/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.type.api.PType;

/**
 * Implementation of TypedElement.
 */
public class BasicTypedElement extends BasicMetaObject
        implements TypedElement, Comparable {

    /**
	 * The name of the primitive field
	 */
	private String name;

	/**
	 * The type of the primitive field
	 */
	private PType type;

    /**
     * The size of the field
     */
    private int size = PType.NOSIZE;


	/**
	 * Builds a new BasicTypedElement object.
	 *
	 * @param name   the name of the primitive field,
     *        pType  the type of the primitive field,
     *        parent the parent of the current object.
	 */
	public BasicTypedElement(String name, PType pType, int size, MetaObject parent) {
        super(parent);
        this.name = name;
        this.type = pType;
        this.size = size;
    }

    public int compareTo(Object o) {
        return name.compareTo(((TypedElement) o).getName());
    }

	/**
	 * Returns the name of the current typed element.
	 *
	 * @return	the string representation of the typed element
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns the type of the field.
	 *
	 * @return	a PType object which defines the type of the current field
	 */
	public PType getType() {
		return type;
	}

    /**
     * Returns the size of the field in case of string, serialized, arrays, etc
     * @return the size for this type
     */
    public int getSize() {
        return size;
    }
}