/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.naming.api;

import java.util.Date;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * Used to encode/decode a composite name to/from a String. For the decode
 * process, there are the typed getter methods (XXX getXXX()). It supposes that
 * the coder knows the String representing the encoded value. For the encode
 * process, there are the setter methods (putXXX(XXX val)) in order to add
 * new sub values to the encoded String. After all putXXX has been done, the
 * encoded value is retrieved using the getStringCode method.
 * @author	P. D?chamboux
 */
public interface PNCStringCoder {
	byte getByte() throws PExceptionNameCoding;

	Byte getObyte() throws PExceptionNameCoding;

	char getChar() throws PExceptionNameCoding;

	Character getOchar() throws PExceptionNameCoding;

	short getShort() throws PExceptionNameCoding;

	Short getOshort() throws PExceptionNameCoding;

	int getInt() throws PExceptionNameCoding;

	Integer getOint() throws PExceptionNameCoding;

	long getLong() throws PExceptionNameCoding;

	Long getOlong() throws PExceptionNameCoding;

	String getString() throws PExceptionNameCoding;

	Date getDate() throws PExceptionNameCoding;

	BigInteger getBigInteger() throws PExceptionNameCoding;

	BigDecimal getBigDecimal() throws PExceptionNameCoding;

	byte[] getByteArray() throws PExceptionNameCoding;

	char[] getCharArray() throws PExceptionNameCoding;

	void putByte(byte val) throws PExceptionNameCoding;

	void putObyte(Byte val) throws PExceptionNameCoding;

	void putChar(char val) throws PExceptionNameCoding;

	void putOchar(Character val) throws PExceptionNameCoding;

	void putShort(short val) throws PExceptionNameCoding;

	void putOshort(Short val) throws PExceptionNameCoding;

	void putInt(int val) throws PExceptionNameCoding;

	void putOint(Integer val) throws PExceptionNameCoding;

	void putLong(long val) throws PExceptionNameCoding;

	void putOlong(Long val) throws PExceptionNameCoding;

	void putString(String val) throws PExceptionNameCoding;

	void putDate(Date val) throws PExceptionNameCoding;

	void putBigInteger(BigInteger val) throws PExceptionNameCoding;

	void putBigDecimal(BigDecimal val) throws PExceptionNameCoding;

	void putByteArray(byte[] val) throws PExceptionNameCoding;

	void putCharArray(char[] val) throws PExceptionNameCoding;

	String getStringCode() throws PExceptionNameCoding;
}
