/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.facility.naming.polymorphid.fos;

// Import clauses required by jorm
import org.objectweb.jorm.api.ConnectionHolder;
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PClassMappingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
//import for the fos mapper
import org.objectweb.jorm.mapper.fos.lib.FosPMappingStructuresManager;
import org.objectweb.jorm.mapper.fos.genclass.FosGenClassProp;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.perseus.connector.ra.fos.FosConnection;
import org.objectweb.perseus.fos.api.FosException;
import java.util.NoSuchElementException;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.Iterator;

public class IdClassMapping
	implements PClassMapping, PClassMappingCtrl, Loggable
     {
	protected static final String MAPPER_NAME = "fos";
	protected static final String PROJECT_NAME = "facility";
	//Fields
	protected static final String className_NAME = "className";
	//Hidden fields
	protected static final String classId_NAME = "classId";

	private final static String[][] SUPER_TYPES =  {
    };

	/** Fully quailied name of the persitent class */
	protected final static String JORM_CLASS_NAME =
		"org.objectweb.jorm.facility.naming.polymorphid.IdClass";
	/** The actual binder which does the binding action */
	private PBinder binder;
	/** The meta object of the current jorm class */
	private Class moc;
	/** The mapper linked to this PClassMapping */
	private PMapper mapper;
    protected final static String FOS_DIRNAME = "jorm_facility/polymorphid/IdClass";
	/**
	 * The logger instance used by this XMapping instance
	 * and the XBinding instances
	 */
	public Logger logger;
	public LoggerFactory loggerFactory;

	public IdClassMapping() {
 	}


	// IMPLEMENTATION OF METHODS FROM THE PClassMappingCtrl INTERFACE

    public void setPNameCoder(PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setPNameCoder(String fn, PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class does'nt contain field reference: " + fn);
    }

    public void setGenClassMapping(PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setGenClassMapping(String fn, PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class does not contain references to generic class");
    }

    public void setProjectName(String projectname) {
    }


	// IMPLEMENTATION OF METHODS FROM THE PClassMapping INTERFACE

	public void init(PMappingCallback m, MetaObject metaclass) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		mapper = m;
		moc = (Class) metaclass;
		PType ptype = mapper.getPTypeSpace().createPType(
			"org.objectweb.jorm.facility.naming.polymorphid.IdClass",
			SUPER_TYPES);
		setPType(ptype);

		((FosPMappingStructuresManager) mapper.getPMappingStructuresManager()).addDirName(JORM_CLASS_NAME, JORM_CLASS_NAME);
	}

	public PMapper getPMapper() {
		return mapper;
	}

	public PBinder getPBinder() {
		return binder;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains references*/
	public PNameCoder getPNameCoder()
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PNameCoder getPNameCoder(String fn)
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class does'nt contain field reference");
	}

	public String getClassName() {
		return JORM_CLASS_NAME;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains another gen class*/
	public PClassMapping getGenClassMapping() throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PClassMapping getGenClassMapping(String fn)
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class does not contain references to generic class");
	}

	public boolean isConform(String mapperName) {
		return MAPPER_NAME.equals(mapperName);
	}

	public String getProjectName() {
		return PROJECT_NAME;
	}

	public MetaObject getMetaInfo() {
		return moc;
	}

	public String[][] getSuperClassNames() {
		return SUPER_TYPES;
	}

	public void setPBinder(PBinder b) {
		binder = b;
	}

	public PBinding createPBinding() throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		PBinding binding = new IdClassBinding();
		try {
			binding.init(this);
		}
		catch(PExceptionProtocol pep) {
		}
		return binding;
	}

    public Iterator getPNameIterator(Object conn) throws PException {
		FosConnection connection;
		if (conn == null)
			throw new PExceptionProtocol("Cannot ${actionName} without a connection");
		else if (conn instanceof FosConnection)
			connection = (FosConnection) conn;
		else
			throw new PExceptionProtocol("Cannot ${actionName} with this kind of connection");
		try {
            return new IdClassResultIterator(connection.scan(FOS_DIRNAME), binder);
        } catch (FosException fe) {
            throw new PExceptionIO(fe);
        }
    }

    public void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException {
    }

	public PType getPType() {
		return binder.getPType();
	}

	public void setPType(PType pt) {
		binder.setPType(pt);
	}


	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}

}
class IdClassResultIterator implements Iterator {
		private Iterator strIt;
		private PBinder pnc;
		IdClassResultIterator(Iterator it, PBinder pnc) {
			this.pnc = pnc;
			strIt = it;
		}
		public boolean hasNext() {
			return strIt.hasNext();
		}
		public Object next() throws NoSuchElementException {
            try {
				return pnc.decodeString((String) strIt.next());
            } catch (Exception e) {
                throw new NoSuchElementException(
                    "Cannot decode to a PName: " + e.getMessage());
            }
		}
		public void remove()
			throws UnsupportedOperationException, IllegalStateException {
            throw new UnsupportedOperationException("");
		}
}
