/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.facility.naming.polymorphid.rdb.sapdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.ConnectionHolder;
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PClassMappingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
//import for the rdb mapper
import org.objectweb.jorm.mapper.rdb.lib.RdbPMappingStructuresManager;
import org.objectweb.jorm.mapper.rdb.adapter.BasicRdbAdapter;
import org.objectweb.transaction.api.ConnectionWrapper;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.NoSuchElementException;
import org.objectweb.jorm.type.api.PTypeSpace;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.Iterator;

public class ClassIdMapping
	implements PClassMapping, PClassMappingCtrl, Loggable
     {
	protected static final String MAPPER_NAME = "rdb.sapdb";
	protected static final String PROJECT_NAME = "facility";
	//Fields
	protected static final String classId_NAME = "classId";
	//Hidden fields
	protected static final String className_NAME = "className";

	private final static String[][] SUPER_TYPES =  {
    };

	/** Fully quailied name of the persitent class */
	protected final static String JORM_CLASS_NAME =
		"org.objectweb.jorm.facility.naming.polymorphid.ClassId";
	/** The actual binder which does the binding action */
	private PBinder binder;
	/** The meta object of the current jorm class */
	private Class moc;
	/** The mapper linked to this PClassMapping */
	private PMapper mapper;
 	/**
	 * The logger instance used by this XMapping instance
	 * and the XBinding instances
	 */
	public Logger logger;
	public LoggerFactory loggerFactory;

	public ClassIdMapping() {
 	}


	// IMPLEMENTATION OF METHODS FROM THE PClassMappingCtrl INTERFACE

    public void setPNameCoder(PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setPNameCoder(String fn, PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class does'nt contain field reference: " + fn);
    }

    public void setGenClassMapping(PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setGenClassMapping(String fn, PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class does not contain references to generic class");
    }

    public void setProjectName(String projectname) {
    }


	// IMPLEMENTATION OF METHODS FROM THE PClassMapping INTERFACE

	public void init(PMappingCallback m, MetaObject metaclass) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		mapper = m;
		moc = (Class) metaclass;
		PType ptype = mapper.getPTypeSpace().createPType(
			"org.objectweb.jorm.facility.naming.polymorphid.ClassId",
			SUPER_TYPES);
		setPType(ptype);

        RdbPMappingStructuresManager msm = (RdbPMappingStructuresManager)
            mapper.getPMappingStructuresManager();
         // Initialization of the structures of the class org.objectweb.jorm.facility.naming.polymorphid.ClassId
        msm.addTableColumn(JORM_CLASS_NAME, "jf_ClassId", "CID", "BIGINT", false, false, true);
        msm.addTableColumn(JORM_CLASS_NAME, "jf_ClassId", "CN", "VARCHAR(254)", false, true, true);
        msm.classDefined(JORM_CLASS_NAME);
	}

	public PMapper getPMapper() {
		return mapper;
	}

	public PBinder getPBinder() {
		return binder;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains references*/
	public PNameCoder getPNameCoder()
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PNameCoder getPNameCoder(String fn)
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class does'nt contain field reference");
	}

	public String getClassName() {
		return JORM_CLASS_NAME;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains another gen class*/
	public PClassMapping getGenClassMapping() throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PClassMapping getGenClassMapping(String fn)
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class does not contain references to generic class");
	}

	public boolean isConform(String mapperName) {
		return MAPPER_NAME.equals(mapperName);
	}

	public String getProjectName() {
		return PROJECT_NAME;
	}

	public MetaObject getMetaInfo() {
		return moc;
	}

	public String[][] getSuperClassNames() {
		return SUPER_TYPES;
	}

	public void setPBinder(PBinder b) {
		binder = b;
	}

	public PBinding createPBinding() throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		PBinding binding = new ClassIdBinding();
		try {
			binding.init(this);
		}
		catch(PExceptionProtocol pep) {
		}
		return binding;
	}

    public Iterator getPNameIterator(Object conn) throws PException {
		Connection connection;
		if (conn==null) {
			throw new PExceptionProtocol("Impossible to $actionName without connection");
		} else if (conn instanceof Connection) {
			connection = (Connection) conn;
		} else if (conn instanceof ConnectionHolder) {
			connection = (Connection) ((ConnectionHolder) conn).getCHConnection();
		} else if (conn instanceof org.objectweb.perseus.persistence.api.ConnectionHolder) {
			try {
				connection = (Connection)
					((org.objectweb.perseus.persistence.api.ConnectionHolder) conn)
					.getCHConnectionForWrite();
			} catch(org.objectweb.perseus.persistence.api.PersistenceException e) {
				throw new PExceptionProtocol(e, "Impossible to ${actionName} with this connection: ");
			}
		} else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else {
			throw new PExceptionProtocol("Impossible to ${actionName} with this connection type.");
		}
		try {
			PreparedStatement ps = connection.prepareStatement("SELECT jf_ClassId.CN FROM jf_ClassId");
            return new ClassIdResultIterator(ps, ps.executeQuery(), binder);
        } catch (java.sql.SQLException e) {
            throw new PExceptionIO(e);
        }
    }

    public void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException {
    }

	public PType getPType() {
		return binder.getPType();
	}

	public void setPType(PType pt) {
		binder.setPType(pt);
	}


	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
 }
    class ClassIdResultIterator
    implements Iterator {
    ResultSet rs;
	PBinder binder;
    Object fieldval;
    boolean hasNext;
    boolean isForw;
	PreparedStatement ps;

    ClassIdResultIterator(PreparedStatement ps, ResultSet r,  PBinder binder)
		throws java.sql.SQLException {
		this.ps = ps;
		this.binder = binder;
        rs = r;
        hasNext = rs.next();
        isForw = true;
    }

	public void finalize() throws Throwable {
		if (rs != null)
			rs.close();
		if (ps != null)
			ps.close();
		super.finalize();
	}

    public boolean hasNext() {
    	boolean res;
        if (isForw) {
            res = hasNext;
        } else {
            isForw = true;
            try {
                hasNext = rs.next();
                res = hasNext;
            } catch(Exception e) {
                e.printStackTrace();
                res = false;
            }
        }
        if (res == false) {
        	try {
				if (rs != null) {
					rs.close();
					rs = null;
				}
				if (ps != null) {
					ps.close();
					ps = null;
				}
			} catch (SQLException e) {
				rs = null;
				ps = null;
			}
        }
        return res;
    }

    public void remove() throws UnsupportedOperationException, IllegalStateException {
        throw new UnsupportedOperationException("");
    }

    public Object next() throws NoSuchElementException {
        try {
            if (!isForw) {
                hasNext = rs.next();
                isForw = true;
            }
            if (!hasNext) {
                throw new NoSuchElementException();
            }
            isForw = false;
            return binder.decodeString(rs.getString(1));
        } catch (Exception e) {
            throw new NoSuchElementException(
                    "Impossible to decode to an ID PName: " + e.getMessage());
        }
    }
}
