/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.lib.MapperJCA;
import org.objectweb.jorm.lib.JormConfigurator;
import org.objectweb.jorm.util.api.Loggable;

import javax.resource.spi.ManagedConnectionFactory;
import javax.resource.ResourceException;
import java.util.Properties;

/**
 * Allocates a new JCA mapper and configures it.
 */
public class JcaMapperFactory implements JormTestMapperFactory {
	public static final String RESOURCE_MCF_CLASSNAME = "mapper.jca.mcf.classname";
	public static final String ADAPTER_PROPERTIES_SETTER = "mapper.jca.mcf.propsetter";
	public static final String DEFAULT_MAPPER_NAME = "fos";

    /**
     * Creates a new mapper for executing a mapping. The specifications of this
     * mapper are passed through the properies parameter.
     * @param prop  The properties specifying the mapper to create.
     * @return  The created mapper.
     * @throws PException
     */
    public PMapper createMapper(Properties prop) throws PException {
        String mcfcn, ascn, jormconf, submapper = null;
		if ((mcfcn = prop.getProperty(RESOURCE_MCF_CLASSNAME)) == null) {
			throw new PException("TEST: ManagedConnectionFactory class name mandatory for configuring a JCA adapter.");
		}
		if ((ascn = prop.getProperty(ADAPTER_PROPERTIES_SETTER)) == null) {
			throw new PException("TEST: JcaAdapterPropertiesSetter class name mandatory for configuring a JCA adapter.");
		}
        if ((jormconf = prop.getProperty(JormTestMapperFactory.JORM_CONFIG_FILE)) == null) {
            throw new PException("TEST: JORM config file mandatory for configuring tests.");
        }
		JormConfigurator jc = new JormConfigurator();
		jc.configure(jormconf);
		jc.configureLog(jormconf);
        PMapper res = new MapperJCA(jc);
        res.setMapperName((submapper == null) ? DEFAULT_MAPPER_NAME : DEFAULT_MAPPER_NAME + "." + submapper);
        ((Loggable) res).setLoggerFactory(jc.getLoggerFactory());
		ManagedConnectionFactory cs = null;
		try {
			cs = (ManagedConnectionFactory) Class.forName(mcfcn).newInstance();
			JcaAdapterPropertiesSetter as = (JcaAdapterPropertiesSetter) Class.forName(ascn).newInstance();
			as.assignProperties(cs, prop, jc.getLoggerFactory());
			res.setConnectionFactory(cs.createConnectionFactory());
		} catch (Exception e) {
			e.printStackTrace();
			if (e instanceof ResourceException) {
				((ResourceException) e).getLinkedException().printStackTrace();
			}
		}
		res.start();
        return res;
    }
}
