/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.mapper.rdb.lib.ConnectionSpecJDBC;
import org.objectweb.jorm.mapper.rdb.lib.MapperJDBC;
import org.objectweb.jorm.lib.JormConfigurator;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.wrapper.p6spy.P6SpyLogger;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Properties;

/**
 * Allocates a new JDBC mapper and configures it.
 */
public class JdbcMapperFactory implements org.objectweb.jorm.JormTestMapperFactory {
    public final static String URL = "jdbc.url";
    public final static String USER = "jdbc.user";
    public final static String PASSWD = "jdbc.passwd";
    public final static String DRIVER = "jdbc.driver";
    public final static String RDB_PRODUCT = "jdbc.product";
    public final static String DEFAULT_MAPPER_NAME = "rdb";

    public final static String SPY_LOGGER_NAME = "org.objectweb.jorm.rdb.sql";

    /**
     * Creates a new mapper for executing a mapping. The specifications of this
     * mapper are passed through the properies parameter.
     * @param prop  The properties specifying the mapper to create.
     * @return  The created mapper.
     * @throws org.objectweb.jorm.api.PException
     */
    public PMapper createMapper(Properties prop) throws PException {
        String url, user, passwd, driver, product, jormconf;
        if ((url = prop.getProperty(URL)) == null) {
            throw new PException("TEST: url mandatory for configuring database.");
        }
        user = prop.getProperty(USER, System.getProperty(USER, null));
        passwd = prop.getProperty(PASSWD, System.getProperty(PASSWD, null));
        if ((driver = prop.getProperty(DRIVER)) == null) {
            throw new PException("TEST: driver mandatory for configuring database.");
        }
        product = prop.getProperty(RDB_PRODUCT, null);
        if ((jormconf = prop.getProperty(JormTestMapperFactory.JORM_CONFIG_FILE)) == null) {
            throw new PException("TEST: JORM config file mandatory for configuring tests.");
        }
        ConnectionSpecJDBC cs = new ConnectionSpecJDBC(url, driver, user, passwd);
		JormConfigurator jcc = new JormConfigurator();
		jcc.configure(jormconf);
		jcc.configureLog(jormconf);
        PMapper res = new MapperJDBC(jcc);
        P6SpyLogger.logger = jcc.getLoggerFactory().getLogger(SPY_LOGGER_NAME);
        P6SpyLogger.level = BasicLevel.DEBUG;
        res.setMapperName((product == null) ? DEFAULT_MAPPER_NAME : DEFAULT_MAPPER_NAME + "." + product);
        ((Loggable) res).setLoggerFactory(jcc.getLoggerFactory());
        res.setConnectionFactory(cs);
		res.start();
        return res;
    }
}
