/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Properties;

/**
 * Allocates a new mapper by delegating its creation to the factory specified
 * into the test configuration file.
 */
public class MapperFactory implements JormTestMapperFactory {
    public final static String FACTORY = "mapper.factory";
    /**
     * Creates a new mapper for executing a mapping. The specifications of this
     * mapper are passed through the properies parameter.
     * @param prop  The properties specifying the mapper to create.
     * @return  The created mapper.
     * @throws org.objectweb.jorm.api.PException
     */
    public PMapper createMapper(Properties prop) throws PException {
        String mfn;
        if ((mfn = prop.getProperty(FACTORY)) == null) {
            throw new PException("TEST: mapper factory mandatory for configuring tests.");
        }
        try {
            return ((JormTestMapperFactory) Class.forName(mfn).newInstance()).createMapper(prop);
        } catch (InstantiationException e) {
            throw new PException(e, "TEST: cannot create mapper factory.");
        } catch (IllegalAccessException e) {
            throw new PException(e, "TEST: cannot create mapper factory.");
        } catch (ClassNotFoundException e) {
            throw new PException(e, "TEST: cannot create mapper factory.");
        }
    }

    public void traceException(Logger logger, Exception e) {
        int i = 0;
        while (e != null) {
            logger.log(BasicLevel.ERROR, (i == 0) ? "" : "Nested level " + i, e);
            i++;
            if (e instanceof PException) {
                e = ((PException) e).getNestedException();
            } else {
                e = null;
            }
        }
    }
}
