/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.lib;

import junit.framework.Test;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import java.io.Serializable;

/**
 * @author	<A HREF="mailto:sebastien.chassande@inrialpes.fr><b>
 *			Sebastien Chassande-Barrioz
 *			</b></A>
 */
public class TestRdbBlobBigObject extends TestRdbBlob implements BlobVerifier {

	private String object = null;

	public TestRdbBlobBigObject() {
		super("Test of RdbBlob class with a big Object");
		System.out.println("Please wait the test takes few minutes...");
	}

	public TestRdbBlobBigObject(String tn) {
		super(tn);
		System.out.println("Please wait the test takes few minutes...");
	}

	public void setUp() {
		try {
			object = "01";
			for (int i = 0; i < 20; i++)
				object += object;
			//System.out.println(object);
			verifier = this;
			blob = new RdbBlob(verifier.getObject());
			assertNotNull("Blob is null", blob);
		}
		catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	protected void tearDown() {
		// Garbage the following objects
		blob = null;
		verifier = null;
	}

	public static void main(String args[]) {
		TestRunner.run(suite());
	}

	public static Test suite() {
		return new TestSuite(TestRdbBlobBigObject.class);
	}


	// IMPLEMENTATION OF THE BlobVerifier INTERFACE //

	public boolean valid(byte aByte, long pos) {
		return true;
	}

	public boolean isSize(long size) {
		return true;
	}

	public boolean isEquals(Object o) {
		return object.equals(o);
	}

	public Serializable getObject() {
		return object;
	}
}

