/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime;

import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.api.PException;
import org.objectweb.util.monolog.api.BasicLevel;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

/**
 * @author	S. Chassandes-Barrioz
 */
public class TestJdbcConnection extends TestRuntimeHelper {

    protected String getLoggerName() {
        return null;
    }

    protected PBinder getBinder(String className) throws Exception {
        return null;
    }

    public TestJdbcConnection(String s) throws Exception {
        super(s);
    }
    public void testGetConnection() throws Exception {
        mapper.closeConnection(mapper.getConnection());
    }

    public void testExistingTable() throws Exception {
        Connection connection = (Connection) mapper.getConnection();
        String name = "JT_EXISTING_TABLE";
        try {
            connection.createStatement().execute("DROP TABLE " + name);
        } catch (SQLException e) {
        }
        logger.log(BasicLevel.DEBUG, "create the table: " + name);
        connection.createStatement().execute(
            "CREATE TABLE " + name + " (f1 INTEGER)");
        ResultSet rs = null;
        boolean existtable = false;
        logger.log(BasicLevel.DEBUG, "Search the table: " + name);
        ArrayList tables = new ArrayList(1);
        try {
            rs = connection.getMetaData()
                    .getTables(null, null, null, null);
            String current = null;
            while (rs.next() && !existtable) {
                current = rs.getString(3);
                logger.log(BasicLevel.DEBUG, "found: " + current);
                logger.log(BasicLevel.DEBUG, "1: " + current);
                tables.add(current);
                existtable = name.equalsIgnoreCase(current);
            }
        } catch (SQLException e) {
            throw new PException(e, "Pb while testing the existence of table " + name);
        } finally {
            if (rs != null) {
                try {
                    rs.close();
                } catch (SQLException e) {
                }
            }
        }
        connection.createStatement().execute("DROP TABLE " + name);
        assertTrue("The table "+ name + " was not found in " + tables, existtable);
    }


}
