/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.basictype;

import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.pobject.basictype.BigDecimalAccessor;
import org.objectweb.jorm.pobject.basictype.BigIntegerAccessor;
import org.objectweb.jorm.pobject.basictype.BooleanAccessor;
import org.objectweb.jorm.pobject.basictype.ByteAccessor;
import org.objectweb.jorm.pobject.basictype.CharAccessor;
import org.objectweb.jorm.pobject.basictype.DoubleAccessor;
import org.objectweb.jorm.pobject.basictype.DateAccessor;
import org.objectweb.jorm.pobject.basictype.FloatAccessor;
import org.objectweb.jorm.pobject.basictype.IntAccessor;
import org.objectweb.jorm.pobject.basictype.LongAccessor;
import org.objectweb.jorm.pobject.basictype.ObooleanAccessor;
import org.objectweb.jorm.pobject.basictype.ObyteAccessor;
import org.objectweb.jorm.pobject.basictype.OcharAccessor;
import org.objectweb.jorm.pobject.basictype.OdoubleAccessor;
import org.objectweb.jorm.pobject.basictype.OfloatAccessor;
import org.objectweb.jorm.pobject.basictype.OintAccessor;
import org.objectweb.jorm.pobject.basictype.OlongAccessor;
import org.objectweb.jorm.pobject.basictype.OshortAccessor;
import org.objectweb.jorm.pobject.basictype.ShortAccessor;
import org.objectweb.jorm.pobject.basictype.StringAccessor;
import org.objectweb.jorm.pobject.basictype.SerializedAccessor;
import org.objectweb.jorm.pobject.basictype.BytearrayAccessor;
import org.objectweb.jorm.pobject.basictype.ChararrayAccessor;

import java.util.Date;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 *
 * @author P.Dechamboux
 */
public class BasicTypeAccessor implements
        BooleanAccessor,
        ObooleanAccessor,
        ByteAccessor,
        ObyteAccessor,
        CharAccessor,
        OcharAccessor,
        ShortAccessor,
        OshortAccessor,
        IntAccessor,
        OintAccessor,
        LongAccessor,
        OlongAccessor,
        FloatAccessor,
        OfloatAccessor,
        DoubleAccessor,
        OdoubleAccessor,
		BigIntegerAccessor,
		BigDecimalAccessor,
		StringAccessor,
        DateAccessor,
        SerializedAccessor,
        BytearrayAccessor,
        ChararrayAccessor,
        PAccessor {

    private boolean boolean_value;
    private byte byte_value;
    private char char_value;
    private short short_value;
    private int int_value;
    private long long_value;
    private float float_value;
    private double double_value;

    private Object object_value;

    BasicTypeAccessor(Object o) {
        object_value = o;
    }

    BasicTypeAccessor(boolean o) {
        boolean_value = o;
    }

    BasicTypeAccessor(byte o) {
        byte_value = o;
    }

    BasicTypeAccessor(char o) {
        char_value = o;
    }

    BasicTypeAccessor(short o) {
        short_value = o;
    }

    BasicTypeAccessor(int o) {
        int_value = o;
    }

    BasicTypeAccessor(long o) {
        long_value = o;
    }

    BasicTypeAccessor(float o) {
        float_value = o;
    }

    BasicTypeAccessor(double o) {
        double_value = o;
    }

    //Accessors to the booleanValue field
    public void paSetBooleanValue(boolean val) throws PException {
        boolean_value = val;
    }

    public boolean paGetBooleanValue() throws PException {
        return boolean_value;
    }

    //Accessors to the obooleanValue field
    public void paSetObooleanValue(Boolean val) throws PException {
        object_value = val;
    }

    public Boolean paGetObooleanValue() throws PException {
        return (Boolean) object_value;
    }

    //Accessors to the byteValue field
    public void paSetByteValue(byte val) throws PException {
        byte_value = val;
    }

    public byte paGetByteValue() throws PException {
        return byte_value;
    }

    //Accessors to the obyteValue field
    public void paSetObyteValue(Byte val) throws PException {
        object_value = val;
    }

    public Byte paGetObyteValue() throws PException {
        return (Byte) object_value;
    }

    //Accessors to the charValue field
    public void paSetCharValue(char val) throws PException {
        char_value = val;
    }

    public char paGetCharValue() throws PException {
        return char_value;
    }

    //Accessors to the ocharValue field
    public void paSetOcharValue(Character val) throws PException {
        object_value = val;
    }

    public Character paGetOcharValue() throws PException {
        return (Character) object_value;
    }

    //Accessors to the shortValue field
    public void paSetShortValue(short val) throws PException {
        short_value = val;
    }

    public short paGetShortValue() throws PException {
        return short_value;
    }

    //Accessors to the oshortValue field
    public void paSetOshortValue(Short val) throws PException {
        object_value = val;
    }

    public Short paGetOshortValue() throws PException {
        return (Short) object_value;
    }

    //Accessors to the intValue field
    public void paSetIntValue(int val) throws PException {
        int_value = val;
    }

    public int paGetIntValue() throws PException {
        return int_value;
    }

    //Accessors to the ointValue field
    public void paSetOintValue(Integer val) throws PException {
        object_value = val;
    }

    public Integer paGetOintValue() throws PException {
        return (Integer) object_value;
    }

    //Accessors to the longValue field
    public void paSetLongValue(long val) throws PException {
        long_value = val;
    }

    public long paGetLongValue() throws PException {
        return long_value;
    }

    //Accessors to the olongValue field
    public void paSetOlongValue(Long val) throws PException {
        object_value = val;
    }

    public Long paGetOlongValue() throws PException {
        return (Long) object_value;
    }

    //Accessors to the floatValue field
    public void paSetFloatValue(float val) throws PException {
        float_value = val;
    }

    public float paGetFloatValue() throws PException {
        return float_value;
    }

    //Accessors to the ofloatValue field
    public void paSetOfloatValue(Float val) throws PException {
        object_value = val;
    }

    public Float paGetOfloatValue() throws PException {
        return (Float) object_value;
    }

    //Accessors to the doubleValue field
    public void paSetDoubleValue(double val) throws PException {
        double_value = val;
    }

    public double paGetDoubleValue() throws PException {
        return double_value;
    }

    //Accessors to the odoubleValue field
    public void paSetOdoubleValue(Double val) throws PException {
        object_value = val;
    }

    public Double paGetOdoubleValue() throws PException {
        return (Double) object_value;
    }

	//Accessors to the bigIntegerValue field
	public void paSetBigIntegerValue(BigInteger val) throws PException {
		object_value = val;
	}

	public BigInteger paGetBigIntegerValue() throws PException {
		return (BigInteger) object_value;
	}

	//Accessors to the bigDecimalValue field
	public void paSetBigDecimalValue(BigDecimal val) throws PException {
		object_value = val;
	}

	public BigDecimal paGetBigDecimalValue() throws PException {
		return (BigDecimal) object_value;
	}

    //Accessors to the stringValue field
    public void paSetStringValue(String val) throws PException {
        object_value = val;
    }

    public String paGetStringValue() throws PException {
        return (String) object_value;
    }


    //Accessors to the dateValue field
    public void paSetDateValue(Date val) throws PException {
        object_value = val;
    }

    public Date paGetDateValue() throws PException {
        return (Date) object_value;
    }


    //Accessors to the serializedValue field
    public void paSetSerializedValue(Serializable val) throws PException {
        object_value = val;
    }

    public Serializable paGetSerializedValue() throws PException {
        return (Serializable) object_value;
    }

    //Accessors to the bytearrayValue field
    public void paSetBytearrayValue(byte[] val) throws PException {
        object_value = val;
    }

    public byte[] paGetBytearrayValue() throws PException {
        return (byte[]) object_value;
    }

    //Accessors to the chararrayValue field
    public void paSetChararrayValue(char[] val) throws PException {
        object_value = val;
    }

    public char[] paGetChararrayValue() throws PException {
        return (char[]) object_value;
    }

    // IMPLEMENTATION OF THE PAccessor INTERFACE //
    //-------------------------------------------//

    /**
     * It returns the memory instance within which the persistent variables are
     * defined.
     * @return	The actual object containing memory persistent variables.
     */
    public Object getMemoryInstance() {
        return this;
    }
}
