/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.namedef;

import org.objectweb.jorm.runtime.TestRuntimeHelper;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.pobject.compositename.CompositeAllTypesIdBinder;
import org.objectweb.jorm.pobject.compositename.CompositeAllTypesIdPNG;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Date;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestAllTypes
        extends TestRuntimeHelper {

    private final static String LOGGER_NAME
            = "test.org.objectweb.jorm.compositename.alltypes";

    public final static String CN
            = "org.objectweb.jorm.pobject.compositename.ACompositeAllTypesId";

    public TestAllTypes(String s) throws Exception {
        super(s);
    }

    protected String getLoggerName() {
        return LOGGER_NAME;
    }

    protected PBinder getBinder(String className) throws Exception {
        if (CN.equals(className)) {
            PBinder binder = new CompositeAllTypesIdBinder();
            binder.setNullPName(new PNG());
            return binder;
        } else {
            throw new Exception("Unmanaged class: " + className);
        }
    }

    public void test() throws Exception {
        logger.log(BasicLevel.DEBUG, "test begin");
        AllTypesAccessor acc2 = new AllTypesAccessor();
        PNG png = new PNG();
        png.byte1 = 1;
        png.byte2 = 2;
        png.bytearray1 = "2".getBytes();
        png.bytearray1 = "3".getBytes();
        png.char1 = '4';
        png.char1 = '5';
        png.oChar1 = new Character('6');
        png.oChar2 = new Character('7');
        png.chararray1 = new char[]{'8'};
        png.chararray2 = new char[]{'9'};
        png.date1 = getTime();
        Thread.sleep(1000);
        png.date2 = getTime();
        png.int1 = 10;
        png.int2 = 11;
        png.long1 = 12;
        png.long2 = 13;
        png.oByte1 = new Byte((byte) 14);
        png.oByte2 = new Byte((byte) 15);
        png.oInt1 = new Integer(16);
        png.oInt2 = new Integer(17);
        png.oLong1 = new Long(18);
        png.oLong2 = new Long(19);
        png.oShort1 = new Short((short) 20);
        png.oShort2 = new Short((short) 21);
        png.short1 = 22;
        png.short2 = 23;
        png.string1 = "24";
        png.string2 = "25";
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CN, png);
        assertEquals("Bad pname encoded value",
            png, pb.getPName().encodeAbstract());

        int f = 43534;
        AllTypesAccessor acc1 = new AllTypesAccessor();
        acc1.f1 = f;
        acc1.setNameValues(pb.getPName());
        acc1.setMyself1(pb.getPClassMapping().getPBinder().getNull());
        acc1.setMyself2(pb.getPName());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);

        AllTypesAccessor acc3 = new AllTypesAccessor();
        acc3.f1 = -f;
        acc3.setNameValues(pb.getPName());
        acc3.setMyself1(pb.getPName());
        acc3.setMyself2(pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);

        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "test end");
    }

    public class PNG implements CompositeAllTypesIdPNG {
        public byte byte1 = -1;
        public byte byte2 = -1;
        public char char1 = '0';
        public char char2 = '0';
        public short short1 = -1;
        public short short2 = -1;
        public int int1 = -1;
        public int int2 = -1;
        public long long1 = -1;
        public long long2 = -1;
        public Byte oByte1 = null;
        public Byte oByte2 = null;
        public Character oChar1 = null;
        public Character oChar2 = null;
        public Short oShort1 = null;
        public Short oShort2 = null;
        public Integer oInt1 = null;
        public Integer oInt2 = null;
        public Long oLong1 = null;
        public Long oLong2 = null;
        public String string1 = null;
        public String string2 = null;
        public Date date1 = null;
        public Date date2 = null;
        public byte[] bytearray1 = null;
        public byte[] bytearray2 = null;
        public char[] chararray1 = null;
        public char[] chararray2 = null;

        public PNG() {
        }

        public boolean equals(Object o) {
            if (o instanceof CompositeAllTypesIdPNG) {
                CompositeAllTypesIdPNG png = (CompositeAllTypesIdPNG) o;
                try {
                    return byte1 == png.pnGetByteF1(null) &&
                    byte2 == png.pnGetByteF2(null) &&
                    char1 == png.pnGetCharF1(null) &&
                    char2 == png.pnGetCharF2(null) &&
                    short1 == png.pnGetShortF1(null) &&
                    short2 == png.pnGetShortF2(null) &&
                    int1 == png.pnGetIntF1(null) &&
                    int2 == png.pnGetIntF2(null) &&
                    long1 == png.pnGetLongF1(null) &&
                    long2 == png.pnGetLongF2(null) &&
                    oByte1 == png.pnGetOByteF1(null) &&
                    oByte2 == png.pnGetOByteF2(null) &&
                    oChar1 == png.pnGetOCharF1(null) &&
                    oChar2 == png.pnGetOCharF2(null) &&
                    oShort1 == png.pnGetOShortF1(null) &&
                    oShort2 == png.pnGetOShortF2(null) &&
                    oInt1 == png.pnGetOIntF1(null) &&
                    oInt2 == png.pnGetOIntF2(null) &&
                    oLong1 == png.pnGetOLongF1(null) &&
                    oLong2 == png.pnGetOLongF2(null) &&
                    string1 == png.pnGetStringF1(null) &&
                    string2 == png.pnGetStringF2(null) &&
                    date1 == png.pnGetDateF1(null) &&
                    date2 == png.pnGetDateF2(null) &&
                    bytearray1 == png.pnGetByteArrayF1(null) &&
                    bytearray2 == png.pnGetByteArrayF2(null) &&
                    chararray1 == png.pnGetCharArrayF1(null) &&
                    chararray2 == png.pnGetCharArrayF2(null);
                } catch (PException e) {
                    e.printStackTrace();
                    return false;
                }
            }
            return false;
        }

        public void setNameValue(AllTypesAccessor a) {
            byte1 = a.byte1Name;
            byte2 = a.byte2Name;
            char1 = a.char1Name;
            char2 = a.char2Name;
            short1 = a.short1Name;
            short2 = a.short2Name;
            int1 = a.int1Name;
            int2 = a.int2Name;
            long1 = a.long1Name;
            long2 = a.long2Name;
            oByte1 = a.oByte1Name;
            oByte2 = a.oByte2Name;
            oChar1 = a.oChar1Name;
            oChar2 = a.oChar2Name;
            oShort1 = a.oShort1Name;
            oShort2 = a.oShort2Name;
            oInt1 = a.oInt1Name;
            oInt2 = a.oInt2Name;
            oLong1 = a.oLong1Name;
            oLong2 = a.oLong2Name;
            string1 = a.string1Name;
            string2 = a.string2Name;
            date1 = a.date1Name;
            date2 = a.date2Name;
            bytearray1 = a.bytearray1Name;
            bytearray2 = a.bytearray2Name;
            chararray1 = a.chararray1Name;
            chararray2 = a.chararray2Name;
        }

        public void setMyself1Value(AllTypesAccessor a) {
            byte1 = a.byte1Myself1;
            byte2 = a.byte2Myself1;
            char1 = a.char1Myself1;
            char2 = a.char2Myself1;
            short1 = a.short1Myself1;
            short2 = a.short2Myself1;
            int1 = a.int1Myself1;
            int2 = a.int2Myself1;
            long1 = a.long1Myself1;
            long2 = a.long2Myself1;
            oByte1 = a.oByte1Myself1;
            oByte2 = a.oByte2Myself1;
            oChar1 = a.oChar1Myself1;
            oChar2 = a.oChar2Myself1;
            oShort1 = a.oShort1Myself1;
            oShort2 = a.oShort2Myself1;
            oInt1 = a.oInt1Myself1;
            oInt2 = a.oInt2Myself1;
            oLong1 = a.oLong1Myself1;
            oLong2 = a.oLong2Myself1;
            string1 = a.string1Myself1;
            string2 = a.string2Myself1;
            date1 = a.date1Myself1;
            date2 = a.date2Myself1;
            bytearray1 = a.bytearray1Myself1;
            bytearray2 = a.bytearray2Myself1;
            chararray1 = a.chararray1Myself1;
            chararray2 = a.chararray2Myself1;
        }

        public void setMyself2Value(AllTypesAccessor a) {
            byte1 = a.byte1Myself2;
            byte2 = a.byte2Myself2;
            char1 = a.char1Myself2;
            char2 = a.char2Myself2;
            short1 = a.short1Myself2;
            short2 = a.short2Myself2;
            int1 = a.int1Myself2;
            int2 = a.int2Myself2;
            long1 = a.long1Myself2;
            long2 = a.long2Myself2;
            oByte1 = a.oByte1Myself2;
            oByte2 = a.oByte2Myself2;
            oChar1 = a.oChar1Myself2;
            oChar2 = a.oChar2Myself2;
            oShort1 = a.oShort1Myself2;
            oShort2 = a.oShort2Myself2;
            oInt1 = a.oInt1Myself2;
            oInt2 = a.oInt2Myself2;
            oLong1 = a.oLong1Myself2;
            oLong2 = a.oLong2Myself2;
            string1 = a.string1Myself2;
            string2 = a.string2Myself2;
            date1 = a.date1Myself2;
            date2 = a.date2Myself2;
            bytearray1 = a.bytearray1Myself2;
            bytearray2 = a.bytearray2Myself2;
            chararray1 = a.chararray1Myself2;
            chararray2 = a.chararray2Myself2;
        }


        // IMPLEMENTATION OF THE CompositeAllTypesIdPNG INTERFACE //
        //--------------------------------------------------------//

        public char[] pnGetCharArrayF2(Object ctx) throws PException {
            return chararray2;
        }

        public char[] pnGetCharArrayF1(Object ctx) throws PException {
            return chararray1;
        }

        public int pnGetIntF2(Object ctx) throws PException {
            return int2;
        }

        public int pnGetIntF1(Object ctx) throws PException {
            return int1;
        }

        public Long pnGetOLongF2(Object ctx) throws PException {
            return oLong2;
        }

        public Long pnGetOLongF1(Object ctx) throws PException {
            return oLong1;
        }

        public short pnGetShortF2(Object ctx) throws PException {
            return short2;
        }

        public short pnGetShortF1(Object ctx) throws PException {
            return short1;
        }

        public byte[] pnGetByteArrayF2(Object ctx) throws PException {
            return bytearray2;
        }

        public byte[] pnGetByteArrayF1(Object ctx) throws PException {
            return bytearray1;
        }

        public byte pnGetByteF2(Object ctx) throws PException {
            return byte2;
        }

        public char pnGetCharF2(Object ctx) throws PException {
            return char2;
        }

        public byte pnGetByteF1(Object ctx) throws PException {
            return byte1;
        }

        public char pnGetCharF1(Object ctx) throws PException {
            return char1;
        }

        public Short pnGetOShortF2(Object ctx) throws PException {
            return oShort2;
        }

        public Short pnGetOShortF1(Object ctx) throws PException {
            return oShort1;
        }

        public Character pnGetOCharF2(Object ctx) throws PException {
            return oChar2;
        }

        public Byte pnGetOByteF2(Object ctx) throws PException {
            return oByte2;
        }

        public long pnGetLongF2(Object ctx) throws PException {
            return long2;
        }

        public Character pnGetOCharF1(Object ctx) throws PException {
            return oChar1;
        }

        public Byte pnGetOByteF1(Object ctx) throws PException {
            return oByte1;
        }

        public String pnGetStringF2(Object ctx) throws PException {
            return string2;
        }

        public long pnGetLongF1(Object ctx) throws PException {
            return long1;
        }

        public String pnGetStringF1(Object ctx) throws PException {
            return string1;
        }

        public Integer pnGetOIntF2(Object ctx) throws PException {
            return oInt2;
        }

        public Date pnGetDateF2(Object ctx) throws PException {
            return date2;
        }

        public Integer pnGetOIntF1(Object ctx) throws PException {
            return oInt1;
        }

        public Date pnGetDateF1(Object ctx) throws PException {
            return date1;
        }
    }
}
